#include <dolfin.h>
#include "mixed.h"

using namespace dolfin;

// boundary value
class DirichletValue : public Expression {

	void eval(Array<double> &values, const Array<double> &x) const {
		values[0] = 1 + x[0] + 2*x[1]*x[1];
	}
};

// Neumann condition
class functionG : public Expression {

	void eval(Array<double>& values, const Array<double>& x) const {
		values[0] = -4*x[1];
	}
};

// Dirichlet boundary
class DirichletBoundary : public SubDomain {

	bool inside(const Array<double> &x, bool on_boundary) const {
		return on_boundary && (x[0] < DOLFIN_EPS || x[0] > 1-DOLFIN_EPS);
	}
};


/*
 *
 * Main
 *
 */
 
int main(void) {

	// Create mesh and function space
	UnitSquareMesh Th(32, 32);
	mixed::FunctionSpace Vh(Th);

	// Define boundary conditions
	DirichletValue uB;
	DirichletBoundary boundary;
	DirichletBC bc(Vh, uB, boundary);
	
	// Define variational forms
	mixed::BilinearForm a(Vh, Vh);
	mixed::LinearForm L(Vh);
	
	Constant f(-6);
	functionG g;
	L.f = f;
	L.g = g;
	
	// Compute solution
	Function u(Vh);
	LinearVariationalProblem problem(a, L, u, bc);
	LinearVariationalSolver solver(problem);
	solver.solve();
	
	// Save solution in VTK format
	File file("mixedPoissonCpp.pvd");
	file << u;
	
	// Plot solution
	plot(u);
	plot(Th);
	interactive();
	
	return 0;
}
