from dolfin import *
set_log_level(PROGRESS)

# Create mesh and define function space
Th = UnitSquareMesh(15,15)
Vh = FunctionSpace(Th, "CG", 1)

# Define boundary conditions
def boundary(x, on_boundary):
    tol = 1E-14           
    return on_boundary and (abs(x[0]) < tol or abs(x[0] - 1) < tol)
    
uB = Expression("1 + x[0] + 2*x[1]*x[1]")
bc = DirichletBC(Vh, uB, boundary)

# Define variational problem
u = TrialFunction(Vh)
v = TestFunction(Vh)
g = Expression("-4*x[1]")
f = Constant(-6)
a = inner(grad(u), grad(v))*dx
L = f*v*dx - g*v*ds

# Compute solution
u = Function(Vh)
problem = LinearVariationalProblem(a, L, u, bc)
solver = LinearVariationalSolver(problem)
solver.parameters["linear_solver"] = "gmres"
solver.parameters["preconditioner"] = "ilu"
gmres_prm = solver.parameters["krylov_solver"]
gmres_prm["absolute_tolerance"] = 1e-7
gmres_prm["relative_tolerance"] = 1e-5
gmres_prm["maximum_iterations"] = 1000
solver.solve()

# L2 error and H1 error
uexact = Expression("1+x[0]*x[0]+2*x[1]*x[1]")
L2 = errornorm(uexact, u, norm_type="L2")
H1 = L2 + errornorm(uexact, u, norm_type="H10")
print "\nL2 error:", L2
print "\nH1 error:", H1, "\n"

# Plot solution and gradient
grad_u = project(grad(u), VectorFunctionSpace(Th, "CG", 1))
plot(u, axes=True)
plot(grad_u)

# Dump solution to file in VTK format (or XML format)
file = File("mixedPoisson.pvd")
file << u
file = File("mixedPoisson.xml")
file << u

# Hold plot
interactive()
